local _, ns = ...
---
--- @module WhoKnows
---
-- @debug @
-- @end - d e b u g @

ns.WhoKnows = {}
local WK = ns.WhoKnows
WK.WKEvFrame = CreateFrame("FRAME", "WKEvFrame")
WK.DebugTable = {}
ns.lineAdded = false
-------------------------------------------------------
local function timestamp()
	local t = date("*t")
	local ampm = t.hour <= 11 and "AM" or "PM"
	local _hour = t.hour
	if t.hour > 12 then _hour = t.hour - 12 end
	return string.format("|cffff8000[%s:%s:%s %s]|r ", _hour, t.min, t.sec, ampm)
end

ns.print = function(...) print("|cff33ff99WhoKnows:|r ", ...) end
ns.debugPrint = (function(...)
	if (ns.WhoKnows.debug ~= true) then return end
	local ts = timestamp()
	local arg1, arg2, arg3 = ...
	local msg = arg1 or "NIL"
	local params = ""
	if msg == "Event" then
		if (arg3 and type(arg3) == "table") then
			for i = 1, #arg3 do
				params = params .. "arg" .. i .. ": " .. (arg3[i] or "nada") .. "  |  "
			end
		elseif (arg3 and type(arg3) == "string") then
			params = arg3
		end
		print(ts .. "|cffff0000WhoKnows Debug Event:|r ", arg2 .. " Returns: " .. params)
	else
		print(ts .. "|cffff0000WhoKnows Debug:|r ", msg)
	end
end)
ns.error = function(...) print("|cffff0000WhoKnows Error:|r " .. string.format(...)) end

-------------------------------------------------------

--- @type ns.Character
ns.Character = {};
ns.Character.__index = ns.Character;

--- @return #ns.Character
function ns.Character:New()
	local self = {};
	setmetatable(self, ns.Character);
	self.name = GetUnitName("player")
	self.race = UnitRace("player")
	self.class = UnitClass("player")
	self.faction = UnitFactionGroup("player")
	self.level = UnitLevel("player")
	self.realm = GetRealmName()
	self.guid = UnitGUID("player")
	self.player = nil
	return self;
end

function ns.Character:SetPlayer()
	self.player = WhoKnows_Data.Skills[self.realm][self.faction][self.name]
	return self.player
end

-------------------------------------------------------
function ns.Character:GetAllCharacters(_realm)
	local results = {}
	for realm in pairs(WhoKnows_Data.Skills) do
		if _realm and _realm == realm then
			for faction in pairs(WhoKnows_Data.Skills[realm]) do
				for player in pairs(WhoKnows_Data.Skills[realm][faction]) do
					table.insert(results, { realm, faction, player })
				end
			end
		end
	end
	return results
end

-------------------------------------------------------
function ns.Character:SetPlayerError(errMsg)
	ns.debugPrint("Can't set ns.Character.player because " .. errMsg .. "  doesn't exist.")
	PlaySoundFile("Sound\\Spells\\AbolishMagic.wav")
end

-------------------------------------------------------
function WK.IsLearnableItem(_L)
	local isLearnable
	if _L and _L:match("Hitem:") then
		local itemAuctionClass = select(6, GetItemInfo(_L))
		if itemAuctionClass == "Miscellaneous" and select(7, GetItemInfo(_L)) ~= "Other" then
			return false
		else
			isLearnable = (itemAuctionClass == "Recipe" or itemAuctionClass == "Miscellaneous")
		end
	end
	return isLearnable or false
end

-------------------------------------------------------
function WK.IsItemLink(_I)
	WK.link = _I
	local isLink
	if type(_I) == "string" then
		isLink = _I:match("|Hitem:")
	end
	return isLink or false
end

-------------------------------------------------------
function WK.GetItemIdFromHyperlink(_h)
	local itemId
	if type(_h) == "string" then
		itemId = _h:match("|Hitem:(%d*)")
	end
	return tonumber(itemId)
end

-------------------------------------------------------



