local _, ns = ...
local WK = ns.WhoKnows
local print = ns.print
local debugPrint = ns.debugPrint
--------------------------------------------------------------------------
WK.PlayerStats = {}

WK.ItemMessages = {
	NONE_CAN_LEARN              = "|cffC79C6ENone of your toons can learn this skill.|r",
	BOP                         = "|cffffe700Unlearnable: BOP item. Only checked current toon.",
	KNOWN                       = " : |cff008500known|r",
	NOT_KNOWN                   = " : |cffC41F3BNOT known|r",
	INCOMPLETE                  = " : |cffffe700Incomplete records. Please load toon and scan professions.|r",
	NEEDS_PREREQ                = " : |cffffe700Prerequisites need to be acquired or met first..|r",
	SKILL_LVL_UNK               = "\n     Skill level unknown",
	SKILL_LVL_TOO_LOW           = "\n     |cffffe700Skill too low to learn (currently: %s needs: %s)|r", --curToonSkillLevel,reqSkillLvl
	LVL_UNK                     = "\n     Toon level unknown",
	LVL_TOO_LOW                 = "\n     |cffffe700Player's level too low. (currently %s needs level %s)|r", -- curToonLevel,reqToonLvl
	UNK_IF_CAN_LEARN            = "\n     Unknown if can learn",
	LEARN_WITH_RESTRICTIONS     = "\n     |cffC41F3BWarning:|cffffe700Class restrictions: May be able to learn but BOP item cannot be equipped.|r",
	UNABLE_TO_LEARN             = "\n     |cffff8000Unable to learn at this time.|r",
	ABLE_TO_LEARN               = "\n     |cff1eff00Able to learn|r",
	NEEDS_ENG_SPEC              = "\n     |cffffe700This toon will need to learn the %s engineering spec to learn this.|r", -- reqEngiSpec
	WRONG_ENG_SPEC              = "\n     |cffffe700This toon has the wrong Engineering spec. Requires %s Engineer.|r", -- reqEngiSpec
}

WK.ItemStatus = {
	UNABLE_TO_LEARN         = 0,
	ABLE_TO_LEARN           = 1,
	UNK_IF_CAN_LEARN        = 2,
	LEARN_WITH_RESTRICTIONS = 3,
	INCOMPLETE_INFO         = 4,
	NEEDS_PREREQ            = 5,
	ALREADY_KNOWN           = 6,
}
-------------------------------------------------------
function WK.PlayerStats:GetToonLearnedStatus(item)
	if ((item.itemClass == "Recipe") and WhoKnows_Data.EnableRecipeCheck) then
		local findEligibleToons = WK.PlayerStats:RecipeKnownBy(item)
		local toonSearchResults = WK.PlayerStats:DetermineLearnability(findEligibleToons, item)
		return toonSearchResults
		--!- return toonSearchResults, isBOP
	elseif (item.isBlueprint) then
		return WK.PlayerStats:SpecialMiscRecipe(item)
	end
end

-------------------------------------------------------
function WK.PlayerStats:DetermineLearnability(toons, item)
	local realm = ns.currentPlayer.realm
	local faction = ns.currentPlayer.faction
	local toonSearchResults = {} -- #table holds toons

	if (WK:tlength(toons) == 0) then
		toonSearchResults["*"] = {}
		if item.isBOP then
			toonSearchResults["*"]["msg"] = WK.ItemMessages.BOP
		else
			toonSearchResults["*"]["msg"] = WK.ItemMessages.NONE_CAN_LEARN
		end
		toonSearchResults["*"]["status"] = 0
		return toonSearchResults
	end

	for toon, isKnown in pairs(toons) do
		toonSearchResults[toon] = {}
		if isKnown == true then
			toonSearchResults[toon]["msg"] = WK.ItemMessages.KNOWN
			toonSearchResults[toon]["status"] = 6
		elseif isKnown == "incomplete" then
			toonSearchResults[toon]["msg"] = WK.ItemMessages.INCOMPLETE
			toonSearchResults[toon]["status"] = 4
		elseif item.needsPrereq then
			toonSearchResults[toon]["msg"] = WK.ItemMessages.NEEDS_PREREQ
			toonSearchResults[toon]["status"] = 5
		elseif item.isNotLearnableYet and item.isBOP then
			toonSearchResults[toon]["msg"] = WK.ItemMessages.NEEDS_PREREQ
			toonSearchResults[toon]["status"] = 5
		else
			local learnStatus = WK.ItemStatus.ABLE_TO_LEARN
			local msg = ""
			local toonSkillLevel = (WhoKnows_Data.Skills[realm][faction][toon]["skillLvls"][item.profession] or -1)
			if item.reqClass and #item.reqClass > 0 then
				local isClass = false
				for _, v in pairs(item.reqClass) do
					isClass = (v == WhoKnows_Data.Skills[realm][faction][toon]["class"])
					if isClass == true then break end
				end
				if isClass == false then
					learnStatus = WK.ItemStatus.LEARN_WITH_RESTRICTIONS
				end
			end
			if ((learnStatus < WK.ItemStatus.LEARN_WITH_RESTRICTIONS) and item.reqSkillLvl > toonSkillLevel) then
				if toonSkillLevel == -1 then
					msg = msg .. "\n     Skill level unknown"
					learnStatus = WK.ItemStatus.UNK_IF_CAN_LEARN
				else
					msg = msg .. WK.ItemMessages.SKILL_LVL_TOO_LOW
					learnStatus = WK.ItemStatus.UNABLE_TO_LEARN
				end
			end
			local toonLevel = (WhoKnows_Data.Skills[realm][faction][toon]["level"] or -1)
			if ((learnStatus < WK.ItemStatus.LEARN_WITH_RESTRICTIONS) and item.reqToonLvl > toonLevel) then
				if toonLevel == -1 then
					msg = msg .. "\n     Toon level unknown"
					learnStatus = WK.ItemStatus.UNK_IF_CAN_LEARN
				else
					msg = msg .. string.format(WK.ItemMessages.LVL_TOO_LOW, toonLevel , item.reqToonLvl)
					learnStatus = WK.ItemStatus.UNABLE_TO_LEARN
				end
			end
			if (item.reqEngiSpec and WhoKnows_Data.Skills[realm][faction][toon]["skillLvls"]["EngineeringSpec"]) then
				if WhoKnows_Data.Skills[realm][faction][toon]["skillLvls"]["EngineeringSpec"] == "Undeclared" then
					msg = msg .. string.format(WK.ItemMessages.NEEDS_ENG_SPEC, item.reqEngiSpec )
					learnStatus = WK.ItemStatus.UNABLE_TO_LEARN
				elseif item.reqEngiSpec ~= WhoKnows_Data.Skills[realm][faction][toon]["skillLvls"]["EngineeringSpec"] then
					msg = msg .. string.format(WK.ItemMessages.WRONG_ENG_SPEC, item.reqEngiSpec )
					learnStatus = WK.ItemStatus.UNABLE_TO_LEARN
				end
			end

			if learnStatus == WK.ItemStatus.UNABLE_TO_LEARN then
				msg = msg .. WK.ItemMessages.UNABLE_TO_LEARN
			elseif learnStatus == WK.ItemStatus.ABLE_TO_LEARN then
				msg = msg .. WK.ItemMessages.ABLE_TO_LEARN
			elseif learnStatus == WK.ItemStatus.UNK_IF_CAN_LEARN then
				msg = msg .. WK.ItemMessages.UNK_IF_CAN_LEARN
			elseif learnStatus == WK.ItemStatus.LEARN_WITH_RESTRICTIONS then
				msg = WK.ItemMessages.LEARN_WITH_RESTRICTIONS
			end

			toonSearchResults[toon]["msg"] = WK.ItemMessages.NOT_KNOWN .. msg
			toonSearchResults[toon]["status"] = learnStatus
		end
	end
	return toonSearchResults
end

-------------------------------------------------------
function WK.PlayerStats:SpecialMiscRecipe(item)
	local resultsTable = {}
	resultsTable[ns.currentPlayer.name] = {}
	if item.isNotLearnableYet then
		resultsTable[ns.currentPlayer.name]["msg"] = WK.ItemMessages.UNABLE_TO_LEARN
		resultsTable[ns.currentPlayer.name]["status"] = WK.ItemStatus.NEEDS_PREREQ
	elseif item.isKnownBySelf then
		resultsTable[ns.currentPlayer.name]["msg"] = WK.ItemMessages.KNOWN
		resultsTable[ns.currentPlayer.name]["status"] = WK.ItemStatus.ALREADY_KNOWN
	else
		resultsTable[ns.currentPlayer.name]["msg"] = WK.ItemMessages.ABLE_TO_LEARN
		resultsTable[ns.currentPlayer.name]["status"] = WK.ItemStatus.ABLE_TO_LEARN
	end
	return resultsTable
end

-------------------------------------------------------
--- Scans all the player's toons in current realm to see if they know a given recipe
-- @param #WhoKnows.items.item A WoW item
-- @return #table A table of toons that know the item.
function WK.PlayerStats:RecipeKnownBy(item)
	local realm = ns.currentPlayer.realm
	local faction = ns.currentPlayer.faction
	local player = ns.currentPlayer.player
	local resultsTable = {}

	--BOP? Then only check current toon
	if item.isBOP then
		if player["skillsKnown"][item.profession] then
			resultsTable[ns.currentPlayer.name] = item.isKnownBySelf or false
		end
		return resultsTable
	end
	-- parse toons looking for item
	for searchPlayer in pairs(WhoKnows_Data.Skills[realm][faction]) do
		local currentPlayer = WhoKnows_Data.Skills[realm][faction][searchPlayer]
		if not currentPlayer["excludeFromScan"] then
			if currentPlayer["skillsKnown"][item.profession] then
				if currentPlayer == player and item.isKnownBySelf then
					resultsTable[searchPlayer] = true
				elseif (WK:tlength(currentPlayer["skillsKnown"][item.profession]) == 0) and currentPlayer["skillLvls"][item.profession] > 0 then
					resultsTable[searchPlayer] = "incomplete"
				else
					local index = currentPlayer["skillsKnown"][item.profession][item.itemName]
					resultsTable[searchPlayer] = (index ~= nil)
				end
			end
		end
	end
	return resultsTable
end

-------------------------------------------------------
--- Gets all of the current player's professions and skill levels and saves to WhoKnows_Data
function WK.PlayerStats:GetPlayerSkillLvls()
	local realm = ns.currentPlayer.realm
	local faction = ns.currentPlayer.faction
	local player = ns.currentPlayer.player

	assert(realm ~= nil, "Error: ns.currentPlayer.realm is nil. Shouldn't be.")
	assert(faction ~= nil, "Error: ns.currentPlayer.faction is nil. Shouldn't be.")
	assert(player ~= nil, "Error: ns.currentPlayer.player is nil. Shouldn't be.")

	local profs = { GetProfessions() }
	for i = 1, #profs do
		if profs[i] then
			local profName, _, profSkillLevel = GetProfessionInfo(profs[i])
			if (profName ~= "Fishing" and profName ~= "Archaeology") then
				player["skillLvls"][profName] = profSkillLevel
				if not player["skillsKnown"][profName] then
					player["skillsKnown"][profName] = {}
				end
			end
		end
	end
	--if engineering, check for Gnomish or Goblin spec
	if player["skillLvls"]["Engineering"] then
		if IsSpellKnown(20219) then -- Gnomish
		player["skillLvls"]["EngineeringSpec"] = "Gnomish"
		elseif IsSpellKnown(20222) then -- Goblin
		player["skillLvls"]["EngineeringSpec"] = "Goblin"
		else
			player["skillLvls"]["EngineeringSpec"] = "Undeclared"
		end
	end
end

-------------------------------------------------------
--- Scans the opened profession and verifies that we have all learned skills
--
function WK.PlayerStats:GetTradeSkills()
	if IsTradeSkillLinked() or IsTradeSkillGuild() then return end
	if not IsTradeSkillReady() then return end
	--TRADE_SKILL_UPDATE fires after UI loads all the trade skills; used if IsTradeSkillReady() returns false
	WK.WKEvFrame:UnregisterEvent("TRADE_SKILL_UPDATE")
	--clear out item cache
	table.wipe(WK.items)
	local tradeskillName = GetTradeSkillLine();
	if tradeskillName == GetSpellInfo(53428) then return end --runeforging

	local player = ns.currentPlayer.player
	if (tradeskillName ~= "First Aid" and tradeskillName ~= "Archaeology" and not player[tradeskillName .. "Link"]) then
		player[tradeskillName .. "Link"] = GetTradeSkillListLink()
	end

	if not player["skillsKnown"][tradeskillName] then WK.PlayerStats:GetPlayerSkillLvls() end

	if GetNumTradeSkills() == 0 then
		ns.debugPrint("GetNumTradeSkills = 0??")
		return
	end
	local skillCount = 0
	local counter = 0
	local subheader
	for i = 1, GetNumTradeSkills() do
		local skillToSearch, lineType, _, isExpanded = GetTradeSkillInfo(i)
		skillToSearch = skillToSearch:lower()
		if lineType == "subheader" then subheader = skillToSearch end
		subheader = subheader or ""
		if (lineType == "header" or lineType == "subheader") and isExpanded == false then
			ExpandTradeSkillSubClass(i)
		elseif lineType ~= "header" and lineType ~= "subheader" then
			skillCount = skillCount + 1
			local index = player["skillsKnown"][tradeskillName][skillToSearch]
			if tradeskillName == "Enchanting" then
				index = player["skillsKnown"][tradeskillName][skillToSearch .. "|" .. subheader]

				if index == nil then
					player["skillsKnown"][tradeskillName][skillToSearch .. "|" .. subheader] = true
					counter = counter + 1
				end
				player["skillsKnown"][tradeskillName][skillToSearch] = nil
			elseif index == nil then -- add it to list
			player["skillsKnown"][tradeskillName][skillToSearch] = subheader
			counter = counter + 1
			debugPrint("Added: " .. skillToSearch .. "  *  " .. subheader)
			end
			-- if type(index) == "boolean" then
			-- player["skillsKnown"][tradeskillName][skillToSearch] = subheader
			-- debugPrint(skillToSearch.." - "..subheader)
			-- end
			-- if subheader then
			-- player["skillsKnown"][tradeskillName][subheader] = nil -- cleanup
			-- end
			-- Mining only uses headers
		end
	end
	ns.debugPrint("WK sees " .. skillCount .. " skills known...")
	ns.debugPrint(" and has " .. WK:tlength(player["skillsKnown"][tradeskillName]) .. " recorded for " .. tradeskillName)
	if counter and counter > 0 then print("Added " .. counter .. " skills to WhoKnows.") end
end

function WK.PlayerStats:UpdateLearnedSkill(skill)
	local _, link, _, _, _, _, profession = GetItemInfo(skill)
	--local profession = select(7, GetItemInfo(skill))
	local itemId = WK.GetItemIdFromHyperlink(link)
	debugPrint("running PlayerStats:UpdateLearnedSkill with new skill: " .. (skill or "nada") .. "\n and profession: " .. (profession or "nada"))
	if not skill or not profession then return end
	local player = ns.currentPlayer.player
	--player["skillsKnown"][profession][skill] = true
	debugPrint("HAve itemid: " .. (itemClass or "nada"))
	if WK.items[itemId] then
		debugPrint("it's in the items cache. will remove.")
		WK.items[itemId] = nil
	else
		debugPrint("it's not in the item cache. oh well.")
	end

	table.wipe(WK.items)
end