local _, ns = ...
local WK = ns.WhoKnows
WK.DebugTable.ttItem = {}
WK.ToolTips = {}

-- Make our own hidden tooltip to parse the data from because the one shown will have extra garbage in the way
WK.tt = CreateFrame("GameTooltip", "itemTooltip", nil, "GameTooltipTemplate");
WK.tt:SetOwner(WorldFrame, "ANCHOR_NONE");

-------------------------------------------------------

local function TooltipSetItem(self)
		if (ns.lineAdded) then return end
		local _, link = self:GetItem()
		if not WK.IsLearnableItem(link) then return end
		local itemId = WK.GetItemIdFromHyperlink(link)
		local tsr = WK.items[itemId]
		local toonSearchResults = {}
		if tsr.isIncomplete then
			toonSearchResults["Warning"] = {}
			toonSearchResults["Warning"]["msg"] = "Status unknown - waiting on server"
		else
			toonSearchResults = WK.PlayerStats:GetToonLearnedStatus(WK.items[itemId])
		end
		if toonSearchResults then
			WK.ToolTips:EditTooltip(self, toonSearchResults, WK.items[itemId][isBOP], itemId)
		end
end

function WK.ToolTips:HookToolTips()
	for _, frame in pairs { GameTooltip, ItemRefTooltip, ShoppingTooltip1, ShoppingTooltip2 } do
		frame:HookScript("OnTooltipCleared", function(tooltip)
			ns.lineAdded = false
		end)
		frame:HookScript("OnTooltipSetItem",TooltipSetItem)
	end
end

local function EnumerateTooltipLines_helper(...)
	local results = {}
	for i = 1, select("#", ...) do
		local region = select(i, ...)
		if region and region:GetObjectType() == "FontString" then
			local text = region:GetText()
			if text then tinsert(results, text) end
		end
	end
	return results;
end

function WK.ToolTips:EnumerateTooltipLines(link)
	WK.tt:ClearLines()
	WK.tt:SetHyperlink(link)
	local results = EnumerateTooltipLines_helper(WK.tt:GetRegions())
	return results
end

-- Make our own hidden tooltip to parse the data from because the one shown will have extra garbage in the way
function WK.ToolTips:PrivateTooltip(link)
	if not WK.tt then
		WK.tt = CreateFrame("GameTooltip", "itemTooltip", nil, "GameTooltipTemplate");
		WK.tt:SetOwner(WorldFrame, "ANCHOR_NONE");
	end
	WK.tt:ClearLines()
	WK.tt:SetHyperlink(link)
	return WK.tt
end

function WK.ToolTips:PrivateTooltipByItemId(itemId)
	if not WK.tt then
		WK.tt = CreateFrame("GameTooltip", "itemTooltip", nil, "GameTooltipTemplate");
		WK.tt:SetOwner(WorldFrame, "ANCHOR_NONE");
	end
	WK.tt:ClearLines()
	WK.tt:SetItemByID(itemId)
	return WK.tt
end

--adds our findings to the tooltip
function WK.ToolTips:EditTooltip(tooltip, toonSearchResults, isBOP, itemId)
	tooltip:AddLine("------------------", 1, .5, 0)
	tooltip:AddLine("WhoKnows:", 1, .5, 0)
	if WK:tlength(toonSearchResults) > 0 then
		for k, v in pairs(toonSearchResults) do
			tooltip:AddLine(k .. v.msg, 0.78, 0.6, 0.43)
		end
	elseif isBOP then
		tooltip:AddLine("Bind on Pickup Recipe", 0.76, 0.12, 0.23)
	end
	tooltip:AddLine("------------------", 1, .5, 0)
	tooltip:AddLine("Item ID: " .. tostring(itemId), 1, 1, 1)

	ns.lineAdded = true
end