local _, ns = ...
local WhoKnows = ns.WhoKnows
WhoKnows.wkframes = {}
local print = ns.print

function WhoKnows.wkframes:Tutorial()
	print("Starting tutorial...")
	local Tutorials = LibStub("CustomTutorials-2.0")
	local txt = [[
	Welcome to Who Knows! In order for Who Knows to know what skills you have already learned, 
	please open your Trade Skills window and open each skill known.
	
	Type /wk in the chatbox to get a list of available commands.
	Please report bugs or problems at http://wow.curseforge.com/addons/who-knows/tickets
	]]
	Tutorials.RegisterTutorials("Who Knows", {
		title = "Who Knows",
		{
			text = txt,
		}
	})
	print("Triggering tutorial")
	Tutorials.TriggerTutorial("Who Knows", 1)
end

function WhoKnows.wkframes:IntroDialog()
	local F = CreateFrame("Frame", "WKDialog", UIParent)
	F:SetHeight(250)
	F:SetWidth(512)
	F:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	F.bg = F:CreateTexture("bg", "BACKGROUND")
	F.bg:SetAllPoints()
	F.bg:SetTexture(0, 0, 0, 1)
	F.title = F:CreateFontString("F_title", "OVERLAY", "GameFontNormalLarge")
	F.title:SetPoint("TOP", F, "TOP", 0, -18)
	F.title:SetText("Who Knows?")

	F.body = F:CreateFontString("F_body", "OVERLAY", "GameFontNormalLeft")
	F.body:SetWordWrap(true)
	F.body:SetWidth(475)
	F.body:SetTextColor(1, 1, 1, 1)
	F.body:SetPoint("TOPLEFT", 20, -50)
	local t = [[Welcome to Who Knows! In order for Who Knows to know what skills you have already learned, please open your Trade Skills window and open each skill known.

Type |cffff8000/wk|r in the chatbox to get a list of available commands.

Please report bugs or problems at http://wow.curseforge.com/addons/who-knows/tickets]]

	F.body:SetText(t)
	local xBtn = CreateFrame("Button", "CloseButton", F, "UIPanelButtonTemplate")
	xBtn:SetPoint("BOTTOMRIGHT", F, -15, 20)
	xBtn:SetText("Close")
	xBtn:SetWidth(100)
	xBtn:SetScript("OnClick", function(self, button, down)
		F:Hide()
		if WhoKnows.ff then WhoKnows.ff:SetAlpha(1) end
	end)
	xBtn:Show()
	return F
end

local function SetRealm(rbName)
	local realm
	local faction
	local toon
	ns.debugPrint("Found Checkbox: " .. rbName)
	realm, faction, toon = strmatch(rbName, "rb_(.*)_(.*)_(.*)")
	if WhoKnows_Data.Skills[realm][faction] == nil then
		WhoKnows_Data.Skills[realm][faction] = {}
	end
	if WhoKnows_Data.Skills[realm][faction][toon] == nil then
		WhoKnows_Data.Skills[realm][faction][toon] = WhoKnows_Data.Skills[realm][toon]
		WhoKnows_Data.Skills[realm][toon] = nil
		ns.debugPrint("Copying WhoKnows_Data.Skills." .. realm .. "." .. toon .. " to WhoKnows_Data.Skills." .. realm .. "." .. faction)
	end
end

function WhoKnows.wkframes:AssignFactionFrame()
	local F = CreateFrame("Frame", nil, UIParent)
	F:SetHeight(300)
	F:SetWidth(512)
	F:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	F.bg = F:CreateTexture("bg", "BACKGROUND")
	F.bg:SetAllPoints()
	F.bg:SetTexture(0, 0, 0, 1)
	F.title = F:CreateFontString("F_title", "OVERLAY", "GameFontNormalLarge")
	F.title:SetPoint("TOP", F, "TOP", 0, -18)
	F.title:SetText("Who Knows?  Faction Assignment")

	F.body = F:CreateFontString("F_body", "OVERLAY", "GameFontNormalLeft")
	F.body:SetPoint("TOP", F, "LEFT", 0, -100)
	F.body:SetWordWrap(true)
	F.body:SetWidth(480)
	F.body:SetTextColor(1, 1, 1, 1)
	F.body:SetPoint("TOPLEFT", 20, -35)
	local t = [[
	Prior to this WhoKnows? didn't account for toons in different factions. 
	
	Setting the correct faction here for your other toons will fix that.
	]]
	F.body:SetText(t)

	F.errMsg = F:CreateFontString("F_errMsg", "OVERLAY", "GameFontNormal")
	F.errMsg:SetTextColor(0.8, 0, 0.12, 1)
	F.errMsg:SetText("Please choose a faction for every character.")
	F.errMsg:SetShown(false)

	--Cancel Button
	local xBtn = CreateFrame("Button", "CloseButton", F, "UIPanelButtonTemplate")
	xBtn:SetPoint("BOTTOMRIGHT", F, -15, 20)
	xBtn:SetText("Cancel")
	xBtn:SetWidth(100)
	xBtn:SetScript("OnClick", function(self, button, down)
		print("Warning: this addon will be disabled until you complete the faction selection. Type /wk setfactions in chat box to try again.")
		F:Hide()
		WhoKnows_Data.NeedsFactionUpdate = true
	end)
	xBtn:Show()
	--Save Button
	local sBtn = CreateFrame("Button", "SaveButton", F, "UIPanelButtonTemplate")
	sBtn:SetPoint("BOTTOMLEFT", F, 15, 20)
	sBtn:SetText("Save")
	sBtn:SetWidth(100)
	sBtn:SetScript("OnClick", function(self, button, down)
		local results = { F:GetChildren() }
		local rbs = {}
		local i
		for i = 1, #results do
			if results[i]:GetObjectType() == "CheckButton" then
				table.insert(rbs, results[i])
			end
		end
		local hasError = false
		for i = 1, #rbs, 2 do
			if not rbs[i]:GetChecked() and not rbs[i + 1]:GetChecked() then
				hasError = true
				rbs[i + 1].Label:SetText("Horde     <--")
			else
				rbs[i + 1].Label:SetText("Horde")
			end
		end
		if hasError then
			F.errMsg:SetPoint("BOTTOMLEFT", 110, 50)
			F.errMsg:SetShown(true)
			return
		else
			F.errMsg:SetShown(false)
		end
		for _, v in pairs(rbs) do
			if v:GetObjectType() == "CheckButton" and v:GetChecked() then
				SetRealm(v:GetName())
			end
		end
		F:Hide()
		if WhoKnows_Data.NeedsFactionUpdate then
			WhoKnows.db:LoadSettings()
			WhoKnows.PlayerStats:GetPlayerSkillLvls()
			WhoKnows.InventoryFrames:LoadItemGlows()
			LibStub("AceConfig-3.0"):RegisterOptionsTable("Who Knows", ns.WhoKnows.options)
			self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Who Knows")
			WhoKnows_Data.NeedsFactionUpdate = nil
		end
	end)
	sBtn:Show()
	if WhoKnows.IDFrame and WhoKnows.IDFrame:IsShown() then
		F:SetAlpha(0)
	end
	return F
end

local function FactionRadioButtons(parent, toonName, realm, xPoint, yPoint)
	local faction = { "Alliance", "Horde" }
	local rb = {}
	for i, v in pairs(faction) do
		rb[i] = CreateFrame("CheckButton", "rb_" .. realm .. "_" .. v .. "_" .. toonName, parent, "UIRadioButtonTemplate")
		rb[i].Label = _G["rb_" .. realm .. "_" .. v .. "_" .. toonName .. "Text"]
		rb[i].Label:SetText(v)
		rb[i].Label:SetWidth(250)
		rb[i].Label:SetJustifyH("LEFT")
		rb[i]:SetPoint("TOPLEFT", xPoint + 150, yPoint)
		--color blue for Alliance, red for Horde
		if i == 1 then rb[i].Label:SetTextColor(0.0, 0.44, 0.87)
		else rb[i].Label:SetTextColor(0.77, 0.12, 0.23)
		end
		xPoint = xPoint + 95
		-- we already fixed the current toon on loadup so skip it
		if v == ns.currentPlayer.faction and toonName == ns.currentPlayer.name then
			rb[i]:SetChecked(true)
		end
		rb[i]:SetScript("OnClick", function(self)
			local button
			for button = 1, 2 do rb[button]:SetChecked(false) end
			self:SetChecked(true)
		end)
	end
	return rb
end

function WhoKnows.wkframes:AssignFaction()
	WhoKnows.ff = WhoKnows.wkframes:AssignFactionFrame()
	local RealmHeaderPointX = 30
	local RealmHeaderPointY = -140
	local PlayerHeaderPointX = 80
	local PlayerHeaderPointY = -170
	for realm, toons in pairs(WhoKnows_Data.Skills) do
		WhoKnows.ff.title = WhoKnows.ff:CreateFontString("WhoKnows.ff_Realm_" .. realm, "OVERLAY", "GameFontNormalLeft")
		WhoKnows.ff.title:SetPoint("TOPLEFT", RealmHeaderPointX, RealmHeaderPointY)
		WhoKnows.ff.title:SetText(realm)
		for toon in pairs(toons) do
			if toon ~= "Alliance" and toon ~= "Horde" then -- We will temporarily have Alliance and Horde in same table as toon names until upgrade is complete
			WhoKnows.ff.player = WhoKnows.ff:CreateFontString("WhoKnows.ff_Player_" .. toon, "OVERLAY", "GameFontNormalLeft")
			WhoKnows.ff.player:SetPoint("TOPLEFT", PlayerHeaderPointX, PlayerHeaderPointY)
			WhoKnows.ff.player:SetText(toon .. ":")
			local rb = FactionRadioButtons(WhoKnows.ff, toon, realm, PlayerHeaderPointX, PlayerHeaderPointY)
			local index
			for index = 1, 2 do rb[index]:Show() end
			PlayerHeaderPointY = PlayerHeaderPointY - 30
			end
		end
		RealmHeaderPointY = PlayerHeaderPointY - 40
		PlayerHeaderPointY = PlayerHeaderPointY - 30
	end
	WhoKnows.ff:SetHeight(math.abs(PlayerHeaderPointY) + 30)
	WhoKnows.ff:Show()
	WhoKnows.ff:Lower()
end